#! /usr/bin/env python
# -*- coding: utf-8 -*-
from tqdm import trange, tqdm
from xml.dom import pulldom
from xml.dom.pulldom import parse
from elasticsearch.client import IngestClient, IndicesClient, SnapshotClient
from elasticsearch import Elasticsearch
from elasticsearch.helpers import scan, parallel_bulk, streaming_bulk
from elasticsearch_dsl import Q, Search, Index, Document, Date, Nested, InnerDoc, Keyword, Text, Integer, Short, Long, Range

# Local modules:
from fiases.fias_download import  uprarFullAdddr
from fiases.fias_info import getUpdateVersion
from fiases.fias_data import Address
from fiases.init_db import createConnection

address = Address(workDir='/Volumes/DATA/downloads/',
                  addressIndex='address')

createConnection(host='localhost',timeout=20)
es = Elasticsearch()

# 1. версия
getUpdateVersion(address)


# 3. распаковка
uprarFullAdddr(address)


# 4. удаление
doc = parse(address.workDir + address.addressDELFullXMLFile)


def genFullHouserData():
    """Удаление не актуальных записей из индекса"""
    for event, node in doc:
        if event == \
                pulldom.START_ELEMENT and node.tagName \
                == address.ADDR_OBJECT_TAG:
            yield {
                "_index": address.IndexName,
                "_type": "_doc",
                "_op_type": address.DELETE_OPER,
                'pipeline': address.PIPELINE_ID,
                "_id": node.getAttribute("AOID"),
                "ao_guid": node.getAttribute("AOGUID"),
                "parent_guid": node.getAttribute("PARENTGUID"),
                "formal_name": node.getAttribute("FORMALNAME"),
                "off_name": node.getAttribute("OFFNAME"),
                "short_name": node.getAttribute("SHORTNAME"),
                "ao_level": node.getAttribute("AOLEVEL"),
                "area_code": node.getAttribute("AREACODE"),
                "city_code": node.getAttribute("CITYCODE"),
                "place_code": node.getAttribute("PLACECODE"),
                "auto_code": node.getAttribute("AUTOCODE"),
                "plan_code": node.getAttribute("PLANCODE"),
                "street_code": node.getAttribute("STREETCODE"),
                "city_ar_code": node.getAttribute("CTARCODE"),
                "extr_code": node.getAttribute("EXTRCODE"),
                "sub_ext_code": node.getAttribute("SEXTCODE"),
                "code": node.getAttribute("CODE"),
                "region_code": node.getAttribute("REGIONCODE"),
                "plain_code": node.getAttribute("PLAINCODE"),
                "postal_code": node.getAttribute("POSTALCODE"),
                "okato": node.getAttribute("OKATO"),
                "oktmo": node.getAttribute("OKTMO"),
                "ifns_fl": node.getAttribute("IFNSFL"),
                "ifns_ul": node.getAttribute("IFNSUL"),
                "terr_ifns_fl": node.getAttribute("TERRIFNSFL"),
                "terr_ifns_ul": node.getAttribute("TERRIFNSUL"),
                "norm_doc": node.getAttribute("NORMDOC"),
                "act_status": node.getAttribute("ACTSTATUS"),
                "live_status": node.getAttribute("LIVESTATUS"),
                "curr_status": node.getAttribute("CURRSTATUS"),
                "oper_status": node.getAttribute("OPERSTATUS"),
                "start_date": node.getAttribute("STARTDATE"),
                "end_date": node.getAttribute("ENDDATE"),
                "bazis_create_date": address.CREATE_DATE_ZERO,
                "bazis_update_date": address.UPDATE_DATE_ZERO,
                "update_date": address.UPDATE_DATE_ZERO,
                "bazis_finish_date": node.getAttribute("ENDDATE")
            }


# print("Удаление адресов...")
# print(address.addressFullXmlFile)
# # print("size: " + str(size(ADDR_INFO_DIC['addressFullXmlSize'], system=si)))
ADDR_CNT = 0
for ok, info in tqdm(parallel_bulk(es,
                                   genFullHouserData(),
                                   raise_on_error=False,
                                   raise_on_exception=False),
                     unit=' адрес',
                     desc=' удалено',
                     total=address.ADDRESS_COUNT):
    if (not ok):
        print(ok, info)
    ADDR_CNT = ADDR_CNT + 1
print(ADDR_CNT)


# 6. снэпшот
# sn = SnapshotClient(es)
# try:
# sn.delete(repository="fias", snapshot=address.IndexName)
# except(Exception):
# pass

sn_body = {
    "indices": address.IndexName,
    "ignore_unavailable": "true",
    "include_global_state": "false",
    "metadata": {
        "taken_by": "fias",
        "taken_because": "backup before update"
    }
}
# sn.create(repository="fias", snapshot=ADDRESS_INDEX, body=sn_body)

# print(sn.get(repository="fias", snapshot="houses"))





from elasticsearch.client import SnapshotClient
from fiases.fias_data import ES
import fiases.fias_data

sn = SnapshotClient(ES)

def register(location="/usr/share/elasticsearch/snapshots"):
    sn_body = {
      "type": "fs",
      "settings": {
        "compress": "true",
            "location": location
        } 
    }
    sn.create_repository(repository="fias", body=sn_body)

def restore():
    ES.indices.delete(index=fiases.fias_data.ADDRESS_INDEX, ignore=[400, 404])
    ES.indices.delete(index=fiases.fias_data.HOUSE_INDEX, ignore=[400, 404])

    sn.restore(repository="fias",
           snapshot="fias_full",
           body={
               "indices": [fiases.fias_data.ADDRESS_INDEX, fiases.fias_data.HOUSE_INDEX]
           })

def restoreIfNotExist():
    if not ES.indices.exists(fiases.fias_data.ADDRESS_INDEX):
        sn.restore(repository="fias",
               snapshot="fias_full",
               body={
                   "indices": [fiases.fias_data.ADDRESS_INDEX, fiases.fias_data.HOUSE_INDEX]
               })
    else:
        pass


def createFullSnapshot():
    try:
        sn.delete(repository="fias", snapshot="fias_full")
    except(Exception):
        pass

    sn_body = {
        "indices": [fiases.fias_data.ADDRESS_INDEX, fiases.fias_data.HOUSE_INDEX],
        "ignore_unavailable": "true",
        "include_global_state": "false",
        "metadata": {
            "taken_by": "fias",
            "taken_because": "backup before update"
        }
    }
    sn.create(repository="fias", snapshot="fias_full", body=sn_body)





#! /usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import shutil
import datetime
from pathlib import Path
from elasticsearch.client import IngestClient
from elasticsearch import Elasticsearch, RequestsHttpConnection
from fiases.init_db import createConnection
from elasticsearch_dsl.connections import connections


def createTmpDir():
    TMPDIR = '/tmp/'
    WRITE_MODE = 0o777
    work_dir = str(Path.home()) + TMPDIR
    if os.path.isdir(work_dir):
        shutil.rmtree(work_dir, ignore_errors=True)
    os.makedirs(work_dir, mode=WRITE_MODE, exist_ok=False)
    return work_dir


DATE_TIME_ZONE = 'T00:00:00Z'
def getDateNow():
    return datetime.datetime.now().strftime("%Y-%m-%d") + DATE_TIME_ZONE


WORK_DIR = createTmpDir()

ADDRESS_INDEX = 'address'
HOUSE_INDEX = 'houses'
HOST = 'es01'
TIME_OUT = 2000
ES = Elasticsearch(host='es01',port=9200, timeout=20000, connection_class=RequestsHttpConnection)
connections.configure(
    default={'hosts': 'es01'},
)
REPOSITORY = 'fias'
INFO_INDEX = 'info'
INDEX_OPER = 'index'
DELETE_OPER = 'delete'
CREATE_DATE_ZERO = getDateNow()
UPDATE_DATE_ZERO = getDateNow()
VERSION_DATE = ''
VERSION_DATE_HOUSE = ''
VERSION_REPORT_DATE = ''
ADDRESS_COUNT = 4000000
AS_ADDR_FILE = 'AS_ADDROBJ_*'
AS_DEL_ADDR_FILE = 'AS_DEL_ADDROBJ_*'
ADDR_OBJECT_TAG = 'Object'
ADDR_PIPELINE_ID = 'addr_drop_pipeline'

HOUSES_COUNT = 70000000
AS_HOUSES_FILE = 'AS_HOUSE_*'
AS_DEL_HOUSES_FILE = 'AS_DEL_HOUSE_*'
HOUSES_OBJECT_TAG = 'House'
HOUSES_PIPELINE_ID = 'house_drop_pipeline'

FIAS_XML_RAR = 'fias_xml.rar'
FIAS_DELTA_XML_RAR = 'fias_delta_xml.rar'
FIAS_URL = 'https://fias.nalog.ru/Public/Downloads/Actual/'
VERSION_TXT_FILE = 'VerDate.txt'

URL_FULL = FIAS_URL + FIAS_XML_RAR
URL_DELTA = FIAS_URL + FIAS_DELTA_XML_RAR


class Address:

    addressFullXmlFile = ''
    addressFullXmlSize = 0

    addressDELFullXMLFile = ''
    addressDELFullXmlSize = 0

    addressDeltaFile = ''
    addressDeltaSize = 0
    addressDeltaRecSize = 0

    def createPreprocessor(self):
        dropPipeline = {
            "description":
            "drop not actulal addresses",
            "processors": [{
                "drop": {
                    "if": "ctx.curr_status  != '0' "
                }
            }, {
                "drop": {
                    "if": "ctx.act_status  != '1'"
                }
            }, {
                "drop": {
                    "if": "ctx.live_status  != '1'"
                }
            }]
        }
        IngestClient(ES).put_pipeline(id=ADDR_PIPELINE_ID,
                                      body=dropPipeline)


class Houses:

    housesFullXmlFile = ''
    housesFullXmlSize = 0

    housesDELFullXMLFile = ''
    housesDELFullXmlSize = 0

    housesDeltaFile = ''
    housesDeltaSize = 0
    housesDeltaRecSize = 0

    def createPreprocessor(self):
        dropPipeline = {
            "description": "drop old houses",
            "processors": [
                {
                    "drop": {
                        "if": """
        //Получаем текущую дату из параметра в формате ISO-8601
        ZonedDateTime zdt = ZonedDateTime.parse(ctx.bazis_update_date);
        long millisDateTime = zdt.toInstant().toEpochMilli();
        ZonedDateTime nowDate =
        ZonedDateTime.ofInstant(Instant.ofEpochMilli(millisDateTime), ZoneId.of("Z")); 

        //Получаем end_date 
        ZonedDateTime endDateZDT = ZonedDateTime.parse(ctx.end_date + "T00:00:00Z");
        long millisDateTimeEndDate = endDateZDT.toInstant().toEpochMilli();
        ZonedDateTime endDate =
        ZonedDateTime.ofInstant(Instant.ofEpochMilli(millisDateTimeEndDate), ZoneId.of("Z")); 

        // Сравниваем даты
          return endDate.isBefore(nowDate)
        """
                    }
                }
            ]
        }
        IngestClient(ES).put_pipeline(id=HOUSES_PIPELINE_ID,
                                      body=dropPipeline)



from urllib import request
import dateutil.parser as parser
import fiases.fias_data


def getUpdateVersion():
    """ Получение даты обновления """
    url = fiases.fias_data.FIAS_URL + fiases.fias_data.VERSION_TXT_FILE 

    request.urlretrieve(url, fiases.fias_data.WORK_DIR + fiases.fias_data.VERSION_TXT_FILE)
    with open(fiases.fias_data.WORK_DIR + fiases.fias_data.VERSION_TXT_FILE, 'r') as f:
        date = f.read()
        fiases.fias_data.VERSION_REPORT_DATE = date
        print("date: ", date)
        VERSION_DATE = parser.parse(date)
        # print(fias_data.VERSION_REPORT_DATE)

        print('Version date: ', VERSION_DATE)
        fiases.fias_data.VERSION_DATE = VERSION_DATE.strptime(str(VERSION_DATE)[:10], '%Y-%m-%d').date()
        fiases.fias_data.VERSION_DATE_HOUSE = str(VERSION_DATE.strptime(str(VERSION_DATE)[:10], '%Y-%m-%d').date()) + fiases.fias_data.DATE_TIME_ZONE



#! /usr/bin/env python
# -*- coding: utf-8 -*-
from tqdm import trange, tqdm
from xml.dom import pulldom
from lxml import etree
from xml.dom.pulldom import parse
from elasticsearch.helpers import parallel_bulk
from elasticsearch_dsl import Search

# Local modules:
import fiases.fias_data
from fiases.fias_download import uprarUpdateAdddr
from fiases.fias_info import getUpdateVersion
from fiases.update_info import findInfoDoc
from fiases.fias_data import ES


def addressUpdate(isDebug, address):

    IS_DEBUG = isDebug

    # 3. распаковка
    uprarUpdateAdddr(address)

    # 4. обновление
    rootDeltaXML = etree.parse(fiases.fias_data.WORK_DIR
                               + address.addressDeltaFile).getroot()
    address.addressDeltaRecSize = len(rootDeltaXML.getchildren())

    doc = parse(fiases.fias_data.WORK_DIR + address.addressDeltaFile)

    def updateIndex():
        """Обновление индекса"""
        for event, node in doc:
            if event == pulldom.START_ELEMENT and node.tagName == fiases.fias_data.ADDR_OBJECT_TAG:
                yield {
                    "_index": fiases.fias_data.ADDRESS_INDEX,
                    "_type": "_doc",
                    "_op_type": fiases.fias_data.INDEX_OPER,
                    'pipeline': fiases.fias_data.ADDR_PIPELINE_ID,
                    "_id": node.getAttribute("AOID"),
                    "ao_guid": node.getAttribute("AOGUID"),
                    "parent_guid": node.getAttribute("PARENTGUID"),
                    "formal_name": node.getAttribute("FORMALNAME"),
                    "off_name": node.getAttribute("OFFNAME"),
                    "short_name": node.getAttribute("SHORTNAME"),
                    "ao_level": node.getAttribute("AOLEVEL"),
                    "area_code": node.getAttribute("AREACODE"),
                    "city_code": node.getAttribute("CITYCODE"),
                    "place_code": node.getAttribute("PLACECODE"),
                    "auto_code": node.getAttribute("AUTOCODE"),
                    "plan_code": node.getAttribute("PLANCODE"),
                    "street_code": node.getAttribute("STREETCODE"),
                    "city_ar_code": node.getAttribute("CTARCODE"),
                    "extr_code": node.getAttribute("EXTRCODE"),
                    "sub_ext_code": node.getAttribute("SEXTCODE"),
                    "code": node.getAttribute("CODE"),
                    "region_code": node.getAttribute("REGIONCODE"),
                    "plain_code": node.getAttribute("PLAINCODE"),
                    "postal_code": node.getAttribute("POSTALCODE"),
                    "okato": node.getAttribute("OKATO"),
                    "oktmo": node.getAttribute("OKTMO"),
                    "ifns_fl": node.getAttribute("IFNSFL"),
                    "ifns_ul": node.getAttribute("IFNSUL"),
                    "terr_ifns_fl": node.getAttribute("TERRIFNSFL"),
                    "terr_ifns_ul": node.getAttribute("TERRIFNSUL"),
                    "norm_doc": node.getAttribute("NORMDOC"),
                    "act_status": node.getAttribute("ACTSTATUS"),
                    "live_status": node.getAttribute("LIVESTATUS"),
                    "curr_status": node.getAttribute("CURRSTATUS"),
                    "oper_status": node.getAttribute("OPERSTATUS"),
                    "start_date": node.getAttribute("STARTDATE"),
                    "end_date": node.getAttribute("ENDDATE"),
                    "bazis_create_date": fiases.fias_data.CREATE_DATE_ZERO,
                    "bazis_update_date": fiases.fias_data.VERSION_DATE,
                    "update_date": node.getAttribute("UPDATEDATE"),
                    "bazis_finish_date": node.getAttribute("ENDDATE")
                }

    if IS_DEBUG:
        for ok, info in tqdm(parallel_bulk(ES, updateIndex(),
                                           raise_on_error=False,
                                           raise_on_exception=False),
                             unit=' address',
                             desc='updated',
                             total=address.addressDeltaRecSize):
            if (not ok):
                if IS_DEBUG:
                    print(ok, info)

    else:
        for ok, info in parallel_bulk(ES, updateIndex(),
                                      raise_on_error=False,
                                      raise_on_exception=False):
            if (not ok):
                print(ok, info)

    return address.addressDeltaRecSize


# addressUpdate(isDebug=True)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
from tqdm import tqdm
from lxml import etree
from xml.dom import pulldom
from xml.dom.pulldom import parse
from elasticsearch.helpers import parallel_bulk

# Local modules:
import fiases.fias_data
from fiases.fias_data import ES
from fiases.index_address import createIndex
from fiases.fias_info import getUpdateVersion
from fiases.fias_download import downloadFull, unRarFullAdddr


def import_address(address):
    fiases.fias_data.createTmpDir()

    # 1. версия
    getUpdateVersion()

    # 2. загрузка
    downloadFull()

    # 3. распаковка
    unRarFullAdddr(address)

    # 4. маппинг
    if (ES.indices.exists(fiases.fias_data.ADDRESS_INDEX)):
        ES.indices.delete(index=fiases.fias_data.ADDRESS_INDEX)

    SHARDS_NUMBER = "1"
    ANALYSIS = {
        "filter": {
            "autocomplete_filter": {
                "type": "edge_ngram",
                "min_gram": 2,
                "max_gram": 20
            },
            "fias_word_delimiter": {
                "type": "word_delimiter",
                "preserve_original": "true",
                "generate_word_parts": "false"
            },
            "fias_stop_filter": {
                "type": "stop",
                "remove_trailing": "true",
                "stopwords_path": "stopwords.txt"
            }
        },
        "analyzer": {
            "autocomplete": {
                "type": "custom",
                "tokenizer": "standard",
                "filter": ["autocomplete_filter"]
            },
            "stop_analyzer": {
                "type": "custom",
                "tokenizer": "whitespace",
                "filter": ["lowercase", "fias_stop_filter", "fias_word_delimiter"]
            }
        }
    }
    SETTINGS = {
        "index": {
            "number_of_shards": SHARDS_NUMBER,
            "number_of_replicas": "0",
            "refresh_interval": "-1",
            "requests": {
                "cache": {
                    "enable": "true"
                }
            },
            "blocks": {
                "read_only_allow_delete": "false"
            },
            "analysis": ANALYSIS
        }
    }

    PROPERTIES = {
        "street_address_suggest": {
            "type": "text",
            "analyzer": "autocomplete",
            "search_analyzer": "stop_analyzer"
        },
        "full_address": {
            "type": "keyword"
        },
        "district_full": {
            "type": "keyword"
        },
        "settlement_full": {
            "type": "keyword"
        },
        "street_full": {
            "type": "keyword"
        },
        "formal_name": {
            "type": "keyword"
        },
        "short_name": {
            "type": "keyword",
        },
        "off_name": {
            "type": "keyword",
        },
        "curr_status": {
            "type": "integer"
        },
        "oper_status": {
            "type": "integer"
        },
        "act_status": {
            "type": "integer",
        },
        "live_status": {
            "type": "integer"
        },
        "cent_status": {
            "type": "integer"
        },
        "ao_guid": {
            "type": "keyword",
        },
        "parent_guid": {
            "type": "keyword",
        },
        "ao_level": {
            "type": "keyword"
        },
        "area_code": {
            "type": "keyword"
        },
        "auto_code": {
            "type": "keyword"
        },
        "city_ar_code": {
            "type": "keyword"
        },
        "city_code": {
            "type": "keyword"
        },
        "street_code": {
            "type": "keyword"
        },
        "extr_code": {
            "type": "keyword"
        },
        "sub_ext_code": {
            "type": "keyword"
        },
        "place_code": {
            "type": "keyword"
        },
        "plan_code": {
            "type": "keyword"
        },
        "plain_code": {
            "type": "keyword",
        },
        "code": {
            "type": "keyword",
        },
        "postal_code": {
            "type": "keyword",
        },
        "region_code": {
            "type": "keyword"
        },
        "street": {
            "type": "keyword",
        },
        "district": {
            "type": "keyword",
        },
        "district_type": {
            "type": "keyword",
        },
        "street_type": {
            "type": "keyword",
        },
        "settlement": {
            "type": "keyword",
        },
        "settlement_type": {
            "type": "keyword",
        },
        "okato": {
            "type": "keyword",
        },
        "oktmo": {
            "type": "keyword",
        },
        "ifns_fl": {
            "type": "keyword",
        },
        "ifns_ul": {
            "type": "keyword",
        },
        "terr_ifns_fl": {
            "type": "keyword",
        },
        "terr_ifns_ul": {
            "type": "keyword",
        },
        "norm_doc": {
            "type": "keyword",
        },
        "start_date": {
            "type": "date"
        },
        "end_date": {
            "type": "date"
        },
        "bazis_finish_date": {
            "type": "date"
        },
        "bazis_create_date": {
            "type": "date"
        },
        "bazis_update_date": {
            "type": "date"
        },
        "update_date": {
            "type": "date"
        },
        "houses": {
            "type": "nested",
            "properties": {
                "houseId": {
                    "type": "keyword",
                },
                "build_num": {
                    "type": "keyword",
                },
                "house_num": {
                    "type": "keyword",
                },
                "str_num": {
                    "type": "keyword",
                },
                "ifns_fl": {
                    "type": "keyword"
                },
                "ifns_ul": {
                    "type": "keyword"
                },
                "postal_code": {
                    "type": "keyword"
                },
                "counter": {
                    "type": "keyword"
                },
                "end_date": {
                    "type": "date"
                },
                "start_date": {
                    "type": "date"
                },
                "update_date": {
                    "type": "date"
                },
                "cad_num": {
                    "type": "keyword"
                },
                "terr_ifns_fl": {
                    "type": "keyword"
                },
                "terr_ifns_ul": {
                    "type": "keyword"
                },
                "okato": {
                    "type": "keyword"
                },
                "oktmo": {
                    "type": "keyword"
                }
            }
        }
    }
    ES.indices.create(index=fiases.fias_data.ADDRESS_INDEX,
                      body={
                          'mappings': {
                              "dynamic": False,
                              "properties": PROPERTIES
                          },
                          "settings": SETTINGS
                      })

    # 6. препроцессор
    address.createPreprocessor()

    # 7. импорт
    doc = parse(fiases.fias_data.WORK_DIR + address.addressFullXmlFile)

    def importFull():
        counter = 0
        for event, node in doc:
            if event == pulldom.START_ELEMENT \
               and node.tagName == fiases.fias_data.ADDR_OBJECT_TAG:
                yield {
                    "_index": fiases.fias_data.ADDRESS_INDEX,
                    "_type": "_doc",
                    "_op_type": fiases.fias_data.INDEX_OPER,
                    'pipeline': fiases.fias_data.ADDR_PIPELINE_ID,
                    "_id": node.getAttribute("AOID"),
                    "ao_guid": node.getAttribute("AOGUID"),
                    "parent_guid": node.getAttribute("PARENTGUID"),
                    "formal_name": node.getAttribute("FORMALNAME"),
                    "off_name": node.getAttribute("OFFNAME"),
                    "short_name": node.getAttribute("SHORTNAME"),
                    "ao_level": node.getAttribute("AOLEVEL"),
                    "area_code": node.getAttribute("AREACODE"),
                    "city_code": node.getAttribute("CITYCODE"),
                    "place_code": node.getAttribute("PLACECODE"),
                    "auto_code": node.getAttribute("AUTOCODE"),
                    "plan_code": node.getAttribute("PLANCODE"),
                    "street_code": node.getAttribute("STREETCODE"),
                    "city_ar_code": node.getAttribute("CTARCODE"),
                    "extr_code": node.getAttribute("EXTRCODE"),
                    "sub_ext_code": node.getAttribute("SEXTCODE"),
                    "code": node.getAttribute("CODE"),
                    "region_code": node.getAttribute("REGIONCODE"),
                    "plain_code": node.getAttribute("PLAINCODE"),
                    "postal_code": node.getAttribute("POSTALCODE"),
                    "okato": node.getAttribute("OKATO"),
                    "oktmo": node.getAttribute("OKTMO"),
                    "ifns_fl": node.getAttribute("IFNSFL"),
                    "ifns_ul": node.getAttribute("IFNSUL"),
                    "terr_ifns_fl": node.getAttribute("TERRIFNSFL"),
                    "terr_ifns_ul": node.getAttribute("TERRIFNSUL"),
                    "norm_doc": node.getAttribute("NORMDOC"),
                    "act_status": node.getAttribute("ACTSTATUS"),
                    "live_status": node.getAttribute("LIVESTATUS"),
                    "curr_status": node.getAttribute("CURRSTATUS"),
                    "oper_status": node.getAttribute("OPERSTATUS"),
                    "start_date": node.getAttribute("STARTDATE"),
                    "end_date": node.getAttribute("ENDDATE"),
                    "bazis_create_date": fiases.fias_data.CREATE_DATE_ZERO,
                    "bazis_update_date": fiases.fias_data.UPDATE_DATE_ZERO,
                    "update_date": node.getAttribute("UPDATEDATE"),
                    "bazis_finish_date": node.getAttribute("ENDDATE")

                }

    ADDR_CNT = 0
    for ok, info in tqdm(parallel_bulk(ES,
                                       importFull(),
                                       raise_on_error=False,
                                       raise_on_exception=False),
                         unit=' адрес',
                         desc=' загружено',
                         total=fiases.fias_data.ADDRESS_COUNT):
        if (not ok):
            print(ok, info)
        ADDR_CNT = ADDR_CNT + 1

    createIndex(isUpdate=False)


# address = fiases.fias_data.Address()
# import_address(address=address)



import re
from urllib import request
from tqdm import tqdm
from rarfile import RarFile
from hurry.filesize import size, si
import fiases.fias_data


class TqdmUpTo(tqdm):
    """ Прогресс загрузки в консоли """

    def update_full(self, b=6_500_000, bsize=1, tsize=6_500_000):
        if tsize is not None:
            self.total = tsize
            self.update(b * bsize - self.n)

    def update_delta(self, b=100_000, bsize=1, tsize=100_000):
        if tsize is not None:
            self.total = tsize
            # will also set self.n = b * bsize
            self.update(b * bsize - self.n)


def downloadUpdate():
    """ Загрузка обновления ФИАС """
    file_name = fiases.fias_data.WORK_DIR + fiases.fias_data.FIAS_DELTA_XML_RAR

    request.urlretrieve(fiases.fias_data.URL_DELTA, filename=file_name)


def downloadFull():
    """ Загрузка полной базы ФИАС """
    file_name = fiases.fias_data.WORK_DIR + fiases.fias_data.FIAS_XML_RAR

    with TqdmUpTo(unit='B',
                  unit_scale=True,
                  miniters=1,
                  desc=fiases.fias_data.URL_FULL.split('/')[-1]) as t:  # all optional kwargs
        request.urlretrieve(fiases.fias_data.URL_FULL,
                            filename=file_name,
                            reporthook=t.update_full,
                            data=None)


def uprarUpdateAdddr(address):
    """Распаковка обновления """
    rf = RarFile(fiases.fias_data.WORK_DIR +
                 fiases.fias_data.FIAS_DELTA_XML_RAR)

    addressMatcher = re.compile(fiases.fias_data.AS_ADDR_FILE)
    for f in rf.infolist():
        if addressMatcher.match(f.filename):
            address.addressDeltaFile = f.filename
            address.addressDeltaSize = f.file_size

    if (address.addressDeltaSize > 0):
        rf.extract(address.addressDeltaFile, fiases.fias_data.WORK_DIR)


def uprarUpdateHouses(houses):
    """Распаковка обновления """
    rf = RarFile(fiases.fias_data.WORK_DIR +
                 fiases.fias_data.FIAS_DELTA_XML_RAR)

    housesMatcher = re.compile(fiases.fias_data.AS_HOUSES_FILE)
    for f in rf.infolist():
        if housesMatcher.match(f.filename):
            houses.housesDeltaFile = f.filename
            houses.housesDeltaSize = f.file_size

    if (houses.housesDeltaSize > 0):

        rf.extract(houses.housesDeltaFile, fiases.fias_data.WORK_DIR)
    else:
        print('files NOT FOUND!')


def uprarDelFullAdddr(address):
    """3.Распаковываем архив с удаленными записями"""
    rf = RarFile(fiases.fias_data.WORK_DIR + fiases.fias_data.FIAS_XML_RAR)
    # rf = RarFile(fle)

    addressMatcher = re.compile(fiases.fias_data.AS_ADDR_FILE)
    addressDelMatcher = re.compile(fiases.fias_data.AS_DEL_ADDR_FILE)
    for f in rf.infolist():
        if addressDelMatcher.match(f.filename):
            address.addressDELFullXMLFile = f.filename
            address.addressDELFullXmlSize = f.file_size

    if (address.addressDELFullXmlSize > 0):
        rf.extract(address.addressDELFullXMLFile, fiases.fias_data.WORK_DIR)


def unRarFullAdddr(address):
    """Распаковка адресов из полной базы ФИАС"""
    rf = RarFile(fiases.fias_data.WORK_DIR + fiases.fias_data.FIAS_XML_RAR)

    addressMatcher = re.compile(fiases.fias_data.AS_ADDR_FILE)
    print('')
    for f in rf.infolist():
        if addressMatcher.match(f.filename):
            address.addressFullXmlFile = f.filename
            address.addressFullXmlSize = f.file_size
    if (address.addressFullXmlSize > 0):
        rf.extract(address.addressFullXmlFile, fiases.fias_data.WORK_DIR)

def unRarFullHouses(houses):
    """Распаковка домов из полной базы ФИАС"""
    rf = RarFile(fiases.fias_data.WORK_DIR + fiases.fias_data.FIAS_XML_RAR)
    housesMatcher = re.compile(fiases.fias_data.AS_HOUSES_FILE)
    for f in rf.infolist():
        if housesMatcher.match(f.filename):
            houses.housesFullXmlFile = f.filename
            houses.housesFullXmlSize = f.file_size
    if (houses.housesFullXmlSize > 0):
        rf.extract(houses.housesFullXmlFile, fiases.fias_data.WORK_DIR)
    else:
        print('Файлы не найдены')



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import datetime
import fiases.fias_data
from fiases.fias_data import Address
from fiases.fias_info import getUpdateVersion
from fiases.address_upd import addressUpdate
from fiases.house_upd import housesUpdate
from fiases.update_info import findInfoDoc
from fiases.fias_download import downloadUpdate 
from fiases.index_address import createIndex
from fiases.snapshot import  createFullSnapshot
from fiases.fias_data import ES


def update(isDebug=False):
    print()
    print('update...')
    print()

    address = fiases.fias_data.Address()
    address.createPreprocessor()
    # 1. версия
    getUpdateVersion()

    print('vesion:')
    print(fiases.fias_data.VERSION_REPORT_DATE)
    print()

    infoDoc = findInfoDoc()

    now = datetime.datetime.now()
    infoDoc.update(update_date=now)

    # 2. загрузка
    downloadUpdate()

    print()
    print('hoses...')
    print()
    houses = fiases.fias_data.Houses()
    houses.createPreprocessor()
    HOUSE_CNT = housesUpdate(isDebug=True, houses=houses)
    infoDoc.update(rec_upd_houses=HOUSE_CNT)

    print()
    print('address...')
    print()
    ADDR_CNT = addressUpdate(isDebug=True, address=address)
    infoDoc.update(rec_upd_address=ADDR_CNT)

    print()
    print('index...')
    print()

    createIndex(isUpdate=True)



    print("finish")


# update(isDebug=True)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
from tqdm import tqdm
from lxml import etree
from pathlib import Path
from fiases.fias_data import ES
from xml.dom import pulldom
from xml.dom.pulldom import parse
from elasticsearch.helpers import parallel_bulk
from elasticsearch.client import IndicesClient

# Local modules:
import fiases.fias_data
from fiases.fias_info import getUpdateVersion
from fiases.fias_download import downloadFull, unRarFullHouses


def import_houses(houses):

    fiases.fias_data.createTmpDir()

    # 1. версия
    getUpdateVersion()

    # 2. загрузка
    downloadFull()

    # 3. распаковка
    unRarFullHouses(houses)

    # 4. маппинг
    if (ES.indices.exists(fiases.fias_data.HOUSE_INDEX)):
        ES.indices.delete(index=fiases.fias_data.HOUSE_INDEX)

    SHARDS_NUMBER = "1"
    SETTINGS = {
        "index": {
            "number_of_shards": SHARDS_NUMBER,
            "number_of_replicas": "0",
            "refresh_interval": "-1",
            "requests": {
                "cache": {
                    "enable": "true"
                }
            },
            "blocks": {
                "read_only_allow_delete": "false"
            }
        }
    }
    PROPERTIES = {
        "ao_guid": {
            "type": "keyword"
        },
        "build_num": {
            "type": "keyword",
        },
        "house_num": {
            "type": "keyword",
        },
        "str_num": {
            "type": "keyword",
        },
        "ifns_fl": {
            "type": "keyword"
        },
        "ifns_ul": {
            "type": "keyword"
        },
        "postal_code": {
            "type": "keyword"
        },
        "counter": {
            "type": "keyword"
        },
        "end_date": {
            "type": "date"
        },
        "start_date": {
            "type": "date"
        },
        "bazis_finish_date": {
            "type": "date"
        },
        "bazis_create_date": {
            "type": "date"
        },
        "bazis_update_date": {
            "type": "date"
        },
        "update_date": {
            "type": "date"
        },
        "cad_num": {
            "type": "keyword"
        },
        "terr_ifns_fl": {
            "type": "keyword"
        },
        "terr_ifns_ul": {
            "type": "keyword"
        },
        "okato": {
            "type": "keyword"
        },
        "oktmo": {
            "type": "keyword"
        }
    }
    ES.indices.create(index=fiases.fias_data.HOUSE_INDEX,
                      body={
                          'mappings': {
                              "dynamic": False,
                              "properties": PROPERTIES
                          },
                          "settings": SETTINGS
                      })

    # 6. препроцессор
    houses.createPreprocessor()

    # 7. импорт
    doc = parse(fiases.fias_data.WORK_DIR + houses.housesFullXmlFile)

    def importFull():
        counter = 0
        for event, node in doc:
            if event == pulldom.START_ELEMENT \
               and node.tagName == fiases.fias_data.HOUSES_OBJECT_TAG:
                yield {
                    "_index": fiases.fias_data.HOUSE_INDEX,
                    "_type": "_doc",
                    "_op_type": fiases.fias_data.INDEX_OPER,
                    'pipeline': fiases.fias_data.HOUSES_PIPELINE_ID,
                    "_id": node.getAttribute("HOUSEID"),
                    "ao_guid": node.getAttribute("AOGUID"),
                    "region_code": node.getAttribute("REGIONCODE"),
                    "start_date": node.getAttribute("STARTDATE"),
                    "end_date": node.getAttribute("ENDDATE"),
                    "update_date": node.getAttribute("UPDATEDATE"),
                    "div_type": node.getAttribute("DIVTYPE"),
                    "postal_code": node.getAttribute("POSTALCODE"),
                    "okato": node.getAttribute("OKATO"),
                    "oktmo": node.getAttribute("OKTMO"),
                    "ifns_fl": node.getAttribute("IFNSFL"),
                    "ifns_ul": node.getAttribute("IFNSUL"),
                    "terr_ifns_fl": node.getAttribute("TERRIFNSFL"),
                    "terr_ifns_ul": node.getAttribute("TERRIFNSUL"),
                    "norm_doc": node.getAttribute("NORMDOC"),
                    "house_num": node.getAttribute("HOUSENUM"),
                    "build_num": node.getAttribute("BUILDNUM"),
                    "str_num": node.getAttribute("STRUCNUM"),
                    "counter": node.getAttribute("COUNTER"),
                    "cad_num": node.getAttribute("CADNUM"),
                    "bazis_create_date": fiases.fias_data.CREATE_DATE_ZERO,
                    "bazis_update_date": fiases.fias_data.UPDATE_DATE_ZERO,
                    "update_date": node.getAttribute("UPDATEDATE"),
                    "bazis_finish_date": node.getAttribute("ENDDATE")
                }

    for ok, info in tqdm(parallel_bulk(ES,
                                       importFull(),
                                       raise_on_error=False,
                                       raise_on_exception=False),
                         unit=' адрес',
                         desc=' загружено',
                         total=fiases.fias_data.HOUSES_COUNT):
        if (not ok):
            print(ok, info)

    IndicesClient(ES).refresh()
    IndicesClient(ES).flush()
    IndicesClient(ES).forcemerge()




# houses = fiases.fias_data.Houses()
# import_houses(houses=houses)



import fiases.fias_data
from fiases.address import import_address
from fiases.houses import import_houses
from fiases.fias_delta_update import update
from fiases.fias_data import ES
from fiases.snapshot import createFullSnapshot
from elasticsearch.client import IndicesClient


def importFull():
    houses = fiases.fias_data.Houses()
    import_houses(houses=houses)
    refreshIndex()

    address = fiases.fias_data.Address()
    import_address(address=address)
    refreshIndex()


def updateFias():
    if not ES.indices.exists(fiases.fias_data.ADDRESS_INDEX):
        print("No ADDR index found. Start import full...")
        importFull()
    else:
        update(isDebug=True)
    createFullSnapshot()
    refreshIndex()
    fiases.fias_data.createTmpDir();


def refreshIndex():
    IndicesClient(ES).refresh()
    IndicesClient(ES).flush()
    IndicesClient(ES).forcemerge()
# updateFias()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
from tqdm import trange, tqdm
from xml.dom import pulldom
from lxml import etree
from xml.dom.pulldom import parse
from elasticsearch.helpers import parallel_bulk

# Local modules:
from fiases.fias_download import uprarUpdateHouses
import fiases.fias_data


def housesUpdate(isDebug, houses):

    IS_DEBUG = isDebug

    # 3. распаковка
    uprarUpdateHouses(houses)

    rootDeltaXML = etree.parse(fiases.fias_data.WORK_DIR
                               + houses.housesDeltaFile).getroot()
    houses.housesDeltaRecSize = len(rootDeltaXML.getchildren())

    doc = parse(fiases.fias_data.WORK_DIR + houses.housesDeltaFile)

    def updateIndex():
        """Обновление индекса"""
        for event, node in doc:
            if event == \
                    pulldom.START_ELEMENT and node.tagName \
                    == fiases.fias_data.HOUSES_OBJECT_TAG:
                yield {
                    "_index": fiases.fias_data.HOUSE_INDEX,
                    "_type": "_doc",
                    "_op_type": fiases.fias_data.INDEX_OPER,
                    'pipeline': fiases.fias_data.HOUSES_PIPELINE_ID,
                    "_id": node.getAttribute("HOUSEID"),
                    "ao_guid": node.getAttribute("AOGUID"),
                    "region_code": node.getAttribute("REGIONCODE"),
                    "div_type": node.getAttribute("DIVTYPE"),
                    "postal_code": node.getAttribute("POSTALCODE"),
                    "okato": node.getAttribute("OKATO"),
                    "oktmo": node.getAttribute("OKTMO"),
                    "ifns_fl": node.getAttribute("IFNSFL"),
                    "ifns_ul": node.getAttribute("IFNSUL"),
                    "terr_ifns_fl": node.getAttribute("TERRIFNSFL"),
                    "terr_ifns_ul": node.getAttribute("TERRIFNSUL"),
                    "norm_doc": node.getAttribute("NORMDOC"),
                    "house_num": node.getAttribute("HOUSENUM"),
                    "build_num": node.getAttribute("BUILDNUM"),
                    "str_num": node.getAttribute("STRUCNUM"),
                    "counter": node.getAttribute("COUNTER"),
                    "cad_num": node.getAttribute("CADNUM"),
                    "start_date": node.getAttribute("STARTDATE"),
                    "end_date": node.getAttribute("ENDDATE"),
                    "update_date": node.getAttribute("UPDATEDATE"),
                    "bazis_create_date": fiases.fias_data.CREATE_DATE_ZERO,
                    "bazis_update_date": fiases.fias_data.VERSION_DATE_HOUSE,
                    "update_date": node.getAttribute("UPDATEDATE"),
                    "bazis_finish_date": node.getAttribute("ENDDATE")
                }

    # ADDR_CNT = 0
    if IS_DEBUG:
        for ok, info in tqdm(parallel_bulk(fiases.fias_data.ES, updateIndex(),
                                           raise_on_error=False,
                                           raise_on_exception=False),
                             unit=' house',
                             desc='updated',
                             total=houses.housesDeltaRecSize):
            # ADDR_CNT = ADDR_CNT + 1
            if (not ok):
                if IS_DEBUG:
                    print(ok, info)

    else:
        for ok, info in parallel_bulk(fiases.fias_data.ES, updateIndex(),
                                      raise_on_error=False,
                                      raise_on_exception=False):
            # ADDR_CNT = ADDR_CNT + 1
            if (not ok):
                print(ok, info)

    return houses.housesDeltaRecSize


# housesUpdate(isDebug=True)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Статистика обновления ФИАС
from elasticsearch_dsl import Index, Document, Date, Keyword, Integer
from fiases.fias_data import ES
import fiases.fias_data

info = Index(fiases.fias_data.INFO_INDEX)
@info.document
class InfoDoc(Document):
    fias_version_date = Keyword()
    update_date = Date()
    rec_upd_address = Integer()
    rec_upd_houses = Integer()

    def save(self, **kwargs):
        return super().save(**kwargs)


def findInfoDoc():
    infoDoc = InfoDoc()
    InfoDoc.init()
    s = InfoDoc.search()
    i = s.query('term', fias_version_date=fiases.fias_data.VERSION_REPORT_DATE)
    infoSearchResult = i.execute(ignore_cache=True)


    if infoSearchResult.hits.total.value:
        infoDoc = InfoDoc.get(infoSearchResult.hits.hits[0]._id)
        pass
    else:
        infoDoc.fias_version_date = fiases.fias_data.VERSION_REPORT_DATE
        infoDoc.save()
    return infoDoc


# InfoDoc.init()

# infoDoc = InfoDoc()
# infoDoc.fias_version_date = VERSION_DATE_REPORT
# infoDoc.count_update_houses = ADDRESS_COUNT
# infoDoc.save()



from elasticsearch_dsl.connections import connections
from elasticsearch import Elasticsearch

IS_DEBUG = False


def createConnection(host, timeout):
    connections.configure(
        default={'hosts': host, 'timeout': timeout},
    )
    es = Elasticsearch()
    return es



import setuptools

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


setuptools.setup(
    name="fiases",
    version="1.8.82",
    author="Oleg Sitnikov",
    author_email="oleg.sitnikov@icloud.com",
    description="FIAS Elasticsearch integration",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/u4097/elasticsearch-fias",
    packages=setuptools.find_packages(),
    install_requires=[
        'elasticsearch>=7.1.0',
        'elasticsearch-dsl>=7.1.0',
        'hurry.filesize>=0.9',
        'rarfile>=3.1',
        'tqdm>=4.40.1',
        'urllib3>=1.25.7',
        'lxml>=4.4.2',
        'py-dateutil>=2.2',
        'requests>=2.22.0'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Database',
        'Topic :: Software Development',
        'Natural Language :: Russian'
    ],
    python_requires='>=3.6',
)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
from tqdm import tqdm
from elasticsearch.helpers import scan
from elasticsearch_dsl import Index, \
    Document, Date, Nested, InnerDoc, Keyword, Text, Integer, Short, Long, Range

import fiases.fias_data
from fiases.fias_data import ES


def createIndex(isUpdate=True):
    # 5. Создаем класс для хранения Адресов
    address = Index(fiases.fias_data.ADDRESS_INDEX)
    address.close()

    houses = Index(fiases.fias_data.HOUSE_INDEX)
    @houses.document
    class House(Document, InnerDoc):
        houseId = Keyword()
        house_num = Keyword()
        build_num = Keyword()
        str_num = Keyword()
        postal_code = Keyword()
        ifns_fl = Keyword()
        ifns_ul = Keyword()
        counter = Keyword()

    @address.document
    class Address(Document):
        ao_guid = Keyword(required=True)
        parent_guid = Keyword()

        act_status = Integer()
        curr_status = Integer()
        live_status = Integer()
        oper_status = Integer()

        formal_name = Keyword()
        off_name = Keyword()
        short_name = Keyword()

        region_code = Keyword()
        ao_level = Keyword()

        area_code = Keyword()
        auto_code = Keyword()
        extr_code = Keyword()
        city_ar_code = Keyword()
        city_code = Keyword()
        street_code = Keyword()
        plan_code = Keyword()
        place_code = Keyword()
        sub_ext_code = Keyword()
        plain_code = Keyword()
        code = Keyword()

        okato = Keyword()
        oktmo = Keyword()

        postal_code = Keyword()
        terr_ifns_fl = Keyword()
        terr_ifns_ul = Keyword()
        ifns_fl = Keyword()
        ifns_ul = Keyword()
        norm_doc = Keyword()

        district = Keyword()
        district_type = Keyword()
        settlement = Keyword()
        settlement_type = Keyword()
        street = Keyword()
        street_type = Keyword()

        start_date = Date()
        end_date = Date()
        update_date = Date()

        street_address_suggest = Text(analyzer="autocomplete")
        full_address = Keyword()
        district_full = Keyword()
        settlement_full = Keyword()
        street_full = Keyword()

        houses = Nested(House)

        def add_house(self, house_num, build_num):
            self.houses.append(
                House(house_num=house_num, build_num=build_num))

        def save(self, **kwargs):
            return super().save(**kwargs)

    Address.init()

    address.open()

    queryAllStreet = {
        "query": {
            "bool": {
                "must": [{
                         "match_all": {}
                         }],
                "filter": {
                    "term": {
                        "ao_level": "7"
                    }
                }
            }
        }

    }

    queryByRegion = {
        "query": {
            "bool": {
                "must": [{
                    "term": {
                        "region_code": {
                            "value": "13"
                        }
                    }
                }],
                "filter": {
                    "term": {
                        "ao_level": "7"
                    }
                }
            }
        }
    }

    update_date = str(fiases.fias_data.VERSION_DATE) \
        + fiases.fias_data.DATE_TIME_ZONE

    queryUpdate = {
        "query": {
            "bool": {
                "must": [{
                         "term": {
                             "update_date": {
                                 "value": update_date
                             }
                         }
                         }],
                "filter": {
                    "term": {
                        "ao_level": "7"
                    }
                }
            }
        }
    }

    # Выбираем все улицы
    if isUpdate:
        print("indexing ...")
        scanResStreet = scan(ES,
                             scroll='1h',
                             query=queryUpdate,
                             index=fiases.fias_data.ADDRESS_INDEX)
        ADDR_UPDATE_CNT = Address.search()\
            .query("term", update_date=update_date)\
            .filter("term", ao_level="7").count()
    else:
        print("Full indexing ...")
        scanResStreet = scan(ES,
                             scroll='1h',
                             query=queryAllStreet,
                             index=fiases.fias_data.ADDRESS_INDEX)

        ADDR_UPDATE_CNT = Address.search()\
            .query("term", ao_level="7").count()

    print("ADDR_UPDATE_CNT: ", ADDR_UPDATE_CNT)
    # Обновляем индеск street_address_suggest
    addrSearch = Address.search()
    homeSearch = House.search()

    print("address: ", Address.search().count())
    print("houses: ", House.search().count())
    houseList = []
    for address in tqdm(scanResStreet,
                        unit=' address',
                        desc='indexed',
                        total=ADDR_UPDATE_CNT):
        # source = address['_source']
        # Получаем улицу
        street = Address.get(address['_id'])

        try:
            # Находим город
            city = addrSearch.query("match",
                                    ao_guid=street.parent_guid).execute()[0]

            if (not city.parent_guid):
                # Для Москвы, Питера и Севастополя регион равен городу.
                district = city
            else:
                # Находим регион
                district = addrSearch.query("match",
                                            ao_guid=city.parent_guid).execute()[0]
        except (Exception):
            print()
            print("Ошибка индексации: ")
            print("city: " + city)
            print()
            print("city.parent_guid: " + str((city.parent_guid == False)))
            print(address['_id'])
            print(street.short_name + "." + street.off_name.lower().strip() +
                  ", " + city.short_name + "." + city.off_name.lower().strip() +
                  ", " + district.short_name.lower().strip() + "." +
                  district.off_name.lower().strip())
            print()
            continue
        else:
            houses = homeSearch.filter("term", ao_guid=street.ao_guid)
            for house in houses.scan():
                houseList.append(house)
        try:
            if (street.postal_code):
                postal_code = street.postal_code + ', '
            else:
                postal_code = ''
            street.update(street_type=street.short_name.strip(),
                          street=street.off_name.strip(),
                          settlement=city.off_name.strip(),
                          settlement_type=city.short_name.strip(),
                          district=district.off_name.strip(),
                          district_type=district.short_name.strip(),
                          street_address_suggest=district.off_name.lower().strip()
                          + " " + city.off_name.lower().strip()
                          + " " + street.off_name.lower().strip(),
                          full_address=postal_code
                          + district.short_name
                          + ' ' + district.off_name + ', '
                          + city.short_name + ' ' + city.off_name + ', '
                          + street.short_name + ' ' + street.off_name,
                          houses=houseList
                          )
            houseList[:] = []
        except(Exception):
            print(house)
    print("finish")



# createIndex(isUpdate=True)
